@@ -3,7 +3,7 @@
 
 import {Tensor} from './tensor.js';
 
-type NonTensorType = never;
+export type NonTensorType = never;
 
 /**
  * Type OnnxValue Represents both tensors and non-tensors value for model's inputs/outputs.
@@ -11,3 +11,8 @@ type NonTensorType = never;
  * NOTE: currently not support non-tensor
  */
 export type OnnxValue = Tensor|NonTensorType;
+
+/**
+ * Type OnnxValueDataLocation represents the location of the data of an OnnxValue.
+ */
+export type OnnxValueDataLocation = Tensor.DataLocation;
