@@ -2,11 +2,15 @@
 // Licensed under the MIT License.
 
 /* eslint-disable @typescript-eslint/no-var-requires, @typescript-eslint/no-require-imports */
+
 // We use "require" instead of "import" here because import statement must be put in top level. Our current code does
-// not allow terser to tree-shaking code as expected because some codes are treated as having side effects.
-// So we import code inside the if-clause to allow terser remove the code safely.
+// not allow bundler to tree-shaking code as expected because some codes are treated as having side effects.
+// So we import code inside the if-clause to allow bundler remove the code safely.
 
 export * from 'onnxruntime-common';
+import * as ort from 'onnxruntime-common';
+export default ort;
+
 import {registerBackend, env} from 'onnxruntime-common';
 import {version} from './version';
 
@@ -16,14 +20,14 @@ if (!BUILD_DEFS.DISABLE_WEBGL) {
 }
 
 if (!BUILD_DEFS.DISABLE_WASM) {
-  const wasmBackend = require('./backend-wasm').wasmBackend;
-  if (!BUILD_DEFS.DISABLE_WEBGPU && typeof navigator !== 'undefined' && navigator.gpu) {
+  const wasmBackend = BUILD_DEFS.DISABLE_TRAINING ? require('./backend-wasm-inference').wasmBackend :
+                                                    require('./backend-wasm-training').wasmBackend;
+  if (!BUILD_DEFS.DISABLE_JSEP) {
     registerBackend('webgpu', wasmBackend, 5);
+    registerBackend('webnn', wasmBackend, 5);
   }
   registerBackend('cpu', wasmBackend, 10);
   registerBackend('wasm', wasmBackend, 10);
-  registerBackend('xnnpack', wasmBackend, 9);
-  registerBackend('webnn', wasmBackend, 9);
 }
 
-env.versions.web = version;
+Object.defineProperty(env.versions, 'web', {value: version, enumerable: true});
