@@ -19,19 +19,28 @@
 //
 // modified to fit the needs of the project
 
-const writeDataToSubAVec4Snippet = (transpose: boolean) => {
+import {DataType} from '../../../../wasm-common';
+import {TensorView} from '../../../tensor-view';
+import {ShapeUtil} from '../../../util';
+import {ProgramInfo, ProgramInputTensorInfoDependency, ProgramUniform} from '../../types';
+import {createTensorShapeVariables, getBroadcastDims, IndicesHelper, inputVariable, internalVariable, outputVariable, ShaderHelper, tensorTypeToWsglStorageType, UniformsArrayType} from '../common';
+import {appendActivationUniforms, appendActivationUniformsData, getActivationSnippet, InternalActivationAttributes} from '../fuse-utils';
+
+import {typeSnippet} from './activation_util';
+
+const writeDataToSubAVec4Snippet = (transpose: boolean, batchDims?: IndicesHelper) => {
   if (transpose) {
     return `
         mm_Asub[inputRow][inputCol] = mm_readA(batch,
           kStart + inputRow,
-          globalRowStart / innerElementSize + inputCol);
+          globalRowStart / innerElementSize + inputCol${batchDims ? ', batchIndices' : ''});
         `;
 
   } else {
     return `
         mm_Asub[inputRow][inputCol] = mm_readA(batch,
           globalRow + innerRow,
-          kStart / innerElementSize + inputCol);
+          kStart / innerElementSize + inputCol${batchDims ? ', batchIndices' : ''});
         `;
   }
 };
@@ -62,8 +71,8 @@ const calculateResultSnippet = (transposeA: boolean, innerElementSize: number) =
 };
 
 export const makeMatMulPackedVec4Source =
-    (workPerThread: number[], workgroupSize: [number, number, number], transposeA = false, tileInner = 32,
-     splitK = false, splitedDimInner = 32, isVectorA = false): string => {
+    (workPerThread: number[], workgroupSize: [number, number, number], type = 'f32', batchDims?: IndicesHelper,
+     transposeA = false, tileInner = 32, splitK = false, splitedDimInner = 32): string => {
       const tileAOuter = workgroupSize[1] * workPerThread[1];
       const tileBOuter = workgroupSize[0] * workPerThread[0];
       const tileAWidth = transposeA ? tileAOuter : tileInner;
@@ -82,8 +91,8 @@ export const makeMatMulPackedVec4Source =
             workPerThread[0]} must be 4.`);
       }
       return `
-var<workgroup> mm_Asub : array<array<vec${innerElementSize}<f32>, ${tileAWidth / innerElementSize}>, ${tileAHight}>;
-var<workgroup> mm_Bsub : array<array<vec4<f32>, ${tileBOuter / workPerThread[0]}>, ${tileInner}>;
+var<workgroup> mm_Asub: array<array<vec${innerElementSize}<${type}>, ${tileAWidth / innerElementSize}>, ${tileAHight}>;
+var<workgroup> mm_Bsub: array<array<vec4<${type}>, ${tileBOuter / workPerThread[0]}>, ${tileInner}>;
 
 const rowPerThread = ${workPerThread[1]};
 const colPerThread = ${workPerThread[0]};
@@ -95,34 +104,36 @@ fn main(@builtin(local_invocation_id) localId : vec3<u32>,
         @builtin(global_invocation_id) globalId : vec3<u32>,
         @builtin(workgroup_id) workgroupId : vec3<u32>) {
   let localRow = i32(localId.y);
-  let tileRow = ${isVectorA ? '0' : 'localRow * rowPerThread'};
+  let tileRow = localRow * rowPerThread;
   let tileCol = i32(localId.x);
 
-  let globalRow = ${isVectorA ? '0' : 'i32(globalId.y) * rowPerThread'};
+  let globalRow =i32(globalId.y) * rowPerThread;
   let globalCol = i32(globalId.x);
   let batch = ${splitK ? '0' : 'i32(globalId.z)'};
+  ${batchDims ? `let batchIndices = ${batchDims.offsetToIndices('u32(batch)')};` : ''}
   let globalRowStart = i32(workgroupId.y) * ${tileAOuter};
 
-  let numTiles = ${splitK ? `${Math.ceil(splitedDimInner / tileInner)}` : '(dimInner - 1) / tileInner + 1'};
+  let num_tiles = ${splitK ? `${Math.ceil(splitedDimInner / tileInner)}` : '(uniforms.dim_inner - 1) / tileInner + 1'};
   var kStart = ${splitK ? `i32(globalId.z) * ${splitedDimInner}` : '0'};
 
-  var acc: array<vec4<f32>, rowPerThread>;
+  var acc: array<vec4<${type}>, rowPerThread>;
 
   // Loop over shared dimension.
   let tileRowB = localRow * ${rowPerThreadB};
-  for (var t = 0; t < numTiles; t = t + 1) {
+  for (var t = 0; t < num_tiles; t = t + 1) {
       // Load one tile of A into local memory.
       for (var innerRow = 0; innerRow < rowPerThread; innerRow = innerRow + 1) {
           let inputRow = tileRow + innerRow;
           let inputCol = tileCol;
-          ${writeDataToSubAVec4Snippet(transposeA)}
+          ${writeDataToSubAVec4Snippet(transposeA, batchDims)}
       }
 
       // Load one tile of B into local memory.
       for (var innerRow = 0; innerRow < ${rowPerThreadB}; innerRow = innerRow + 1) {
           let inputRow = tileRowB + innerRow;
           let inputCol = tileCol;
-          mm_Bsub[inputRow][inputCol] = mm_readB(batch, kStart + inputRow, globalCol);
+          mm_Bsub[inputRow][inputCol] = mm_readB(batch, kStart + inputRow, globalCol${
+          batchDims ? ', batchIndices' : ''});
       }
       kStart = kStart + tileInner;
       workgroupBarrier();
@@ -146,19 +157,19 @@ fn main(@builtin(local_invocation_id) localId : vec3<u32>,
 }`;
     };
 
-const writeDataToSubASnippet = (transpose: boolean) => {
+const writeDataToSubASnippet = (transpose: boolean, batchDims?: IndicesHelper) => {
   if (transpose) {
     return `
             mm_Asub[inputRow][inputCol] = mm_readA(batch,
               kStart + inputRow,
-              globalRowStart + inputCol);
+              globalRowStart + inputCol${batchDims ? ', batchIndices' : ''});
             `;
 
   } else {
     return `
             mm_Asub[inputRow][inputCol] = mm_readA(batch,
               globalRowStart + inputRow,
-              kStart + inputCol);
+              kStart + inputCol${batchDims ? ', batchIndices' : ''});
             `;
   }
 };
@@ -169,8 +180,9 @@ const readDataFromSubASnippet = (transposeA: boolean) =>
 // sequentialAccessByThreads means sequential data in memory is accessed by
 // threads, instead of a single thread (default behavior).
 export const makeMatMulPackedSource =
-    (workPerThread: number[], workgroupSize: [number, number, number], transposeA = false, tileInner = 32,
-     splitK = false, splitedDimInner = 32, sequentialAccessByThreads = false): string => {
+    (workPerThread: number[], workgroupSize: [number, number, number], type = 'f32', batchDims?: IndicesHelper,
+     transposeA = false, tileInner = 32, splitK = false, splitedDimInner = 32,
+     sequentialAccessByThreads = false): string => {
       const tileAOuter = workPerThread[1] * workgroupSize[1];
       const tileBOuter = workPerThread[0] * workgroupSize[0];
       const tileAWidth = transposeA ? tileAOuter : tileInner;
@@ -193,11 +205,11 @@ export const makeMatMulPackedSource =
     let globalColStart = i32(workgroupId.x) * ${tileBOuter};
 
     // Loop over shared dimension.
-    for (var t = 0; t < numTiles; t = t + 1) {
+    for (var t = 0; t < num_tiles; t = t + 1) {
       // Load one tile of A into local memory.
       for (var inputRow = localRow; inputRow < ${tileAHight}; inputRow = inputRow + ${workgroupSize[1]}) {
         for (var inputCol = localCol; inputCol < ${tileAWidth}; inputCol = inputCol + ${workgroupSize[0]}) {
-          ${writeDataToSubASnippet(transposeA)}
+          ${writeDataToSubASnippet(transposeA, batchDims)}
         }
       }
       // Load one tile of B into local memory.
@@ -205,14 +217,14 @@ export const makeMatMulPackedSource =
             for (var inputCol = localCol; inputCol < ${tileBOuter}; inputCol = inputCol + ${workgroupSize[0]}) {
           mm_Bsub[inputRow][inputCol] = mm_readB(batch,
             kStart + inputRow,
-            globalColStart + inputCol);
+            globalColStart + inputCol${batchDims ? ', batchIndices' : ''});
         }
       }
       kStart = kStart + tileInner;
       workgroupBarrier();
 
       // Compute acc values for a single thread.
-      var BCached : array<f32, colPerThread>;
+      var BCached : array<${type}, colPerThread>;
       for (var k = 0; k < tileInner; k = k + 1) {
         for (var inner = 0; inner < colPerThread; inner = inner + 1) {
           BCached[inner] = mm_Bsub[k][localCol + inner * ${workgroupSize[0]}];
@@ -249,13 +261,13 @@ let tileRowA = i32(localId.y) * ${rowPerThreadA};
 let tileColA = i32(localId.x) * ${colPerThreadA};
 let tileRowB = i32(localId.y) * ${rowPerThreadB};
 // Loop over shared dimension.
-for (var t = 0; t < numTiles; t = t + 1) {
+for (var t = 0; t < num_tiles; t = t + 1) {
   // Load one tile of A into local memory.
   for (var innerRow = 0; innerRow < ${rowPerThreadA}; innerRow = innerRow + 1) {
     for (var innerCol = 0; innerCol < ${colPerThreadA}; innerCol = innerCol + 1) {
       let inputRow = tileRowA + innerRow;
       let inputCol = tileColA + innerCol;
-      ${writeDataToSubASnippet(transposeA)}
+      ${writeDataToSubASnippet(transposeA, batchDims)}
     }
   }
 
@@ -266,14 +278,14 @@ for (var t = 0; t < numTiles; t = t + 1) {
       let inputCol = tileCol + innerCol;
       mm_Bsub[inputRow][inputCol] = mm_readB(batch,
         kStart + inputRow,
-        globalCol + innerCol);
+        globalCol + innerCol${batchDims ? ', batchIndices' : ''});
     }
   }
   kStart = kStart + tileInner;
   workgroupBarrier();
 
   // Compute acc values for a single thread.
-  var BCached : array<f32, colPerThread>;
+  var BCached : array<${type}, colPerThread>;
   for (var k = 0; k < tileInner; k = k + 1) {
     for (var inner = 0; inner < colPerThread; inner = inner + 1) {
       BCached[inner] = mm_Bsub[k][tileCol + inner];
@@ -299,8 +311,8 @@ for (var innerRow = 0; innerRow < rowPerThread; innerRow = innerRow + 1) {
 `;
 
       return `
-  var<workgroup> mm_Asub : array<array<f32, ${tileAWidth}>, ${tileAHight}>;
-  var<workgroup> mm_Bsub : array<array<f32, ${tileBOuter}>, ${tileInner}>;
+  var<workgroup> mm_Asub : array<array<${type}, ${tileAWidth}>, ${tileAHight}>;
+  var<workgroup> mm_Bsub : array<array<${type}, ${tileBOuter}>, ${tileInner}>;
   const rowPerThread = ${workPerThread[1]};
   const colPerThread = ${workPerThread[0]};
   const tileInner = ${tileInner};
@@ -310,18 +322,182 @@ fn main(@builtin(local_invocation_id) localId : vec3<u32>,
         @builtin(global_invocation_id) globalId : vec3<u32>,
         @builtin(workgroup_id) workgroupId : vec3<u32>) {
     let batch = ${splitK ? '0' : 'i32(globalId.z)'};
-    let numTiles = ${splitK ? `${Math.ceil(splitedDimInner / tileInner)}` : '(dimInner - 1) / tileInner + 1'};
+    ${batchDims ? `let batchIndices = ${batchDims.offsetToIndices('u32(batch)')};` : ''}
+    let num_tiles = ${
+          splitK ? `${Math.ceil(splitedDimInner / tileInner)}` : '(uniforms.dim_inner - 1) / tileInner + 1'};
     var kStart = ${splitK ? `i32(globalId.z) * ${splitedDimInner}` : '0'};
 
-    var acc : array<array<f32, colPerThread>, rowPerThread>;
-
-    // Without this initialization strange values show up in acc.
-    for (var innerRow = 0; innerRow < rowPerThread; innerRow = innerRow + 1) {
-      for (var innerCol = 0; innerCol < colPerThread; innerCol = innerCol + 1) {
-        acc[innerRow][innerCol] = 0.0;
-      }
-    }
+    var acc : array<array<${type}, colPerThread>, rowPerThread>;
     ${matmulSnippet}
   }
 `;
     };
+
+const matMulReadWriteFnSource =
+    (component: number, hasBias: boolean, applyActivation: string, variables: IndicesHelper[],
+     batchShapes: Array<readonly number[]>, isChannelsLast = false): string => {
+      const [batchAShape, batchBShape, batchShape] = batchShapes;
+      const [batchVariable, aVariable, bVariable, outputVariable] = variables;
+      const broadCastADims = getBroadcastDims(batchAShape, batchShape);
+      const broadCastBDims = getBroadcastDims(batchBShape, batchShape);
+      const dataType = tensorTypeToWsglStorageType(variables[0].type.tensor);
+      const getAIndices = () => {
+        const aRank = aVariable.rank;
+        const batchRank = batchVariable.rank;
+        let resStr = `var aIndices: ${aVariable.type.indices};`;
+        for (let i = aRank - 2 - 1, j = batchRank - 1; i >= 0; i--, j--) {
+          resStr += `\naIndices[${i}] = ${batchRank > 1 ? `batchIndices[${j}]` : 'batchIndices'};`;
+        }
+        broadCastADims.forEach(i => {
+          resStr += `\naIndices[${i}] = 0;`;
+        });
+        resStr += `\naIndices[${aRank - 2}] = u32(row);
+                   aIndices[${aRank - 1}] = u32(colIn);`;
+        return resStr;
+      };
+      const getBIndices = () => {
+        const bRank = bVariable.rank;
+        const batchRank = batchVariable.rank;
+        let resStr = `var bIndices: ${bVariable.type.indices};`;
+        for (let i = bRank - 2 - 1, j = batchRank - 1; i >= 0; i--, j--) {
+          resStr += `\nbIndices[${i}] = ${batchRank > 1 ? `batchIndices[${j}]` : 'batchIndices'};`;
+        }
+        broadCastBDims.forEach(i => {
+          resStr += `\nbIndices[${i}] = 0;`;
+        });
+        resStr += `\nbIndices[${bRank - 2}] = u32(row);
+                   bIndices[${bRank - 1}] = u32(colIn);`;
+        return resStr;
+      };
+      const source = `
+    fn mm_readA(batch: i32, row: i32, colIn: i32, batchIndices: ${batchVariable.type.indices}) -> ${
+          typeSnippet(component, dataType)} {
+      var value = ${typeSnippet(component, dataType)}(0.0);
+      let col = colIn * ${component};
+      if(row < uniforms.dim_a_outer && col < uniforms.dim_inner)
+      {
+        ${getAIndices()}
+        value = ${aVariable.getByIndices('aIndices')};
+      }
+      return value;
+    }
+
+    fn mm_readB(batch: i32, row: i32, colIn: i32, batchIndices: ${batchVariable.type.indices}) -> ${
+          typeSnippet(component, dataType)} {
+      var value = ${typeSnippet(component, dataType)}(0.0);
+      let col = colIn * ${component};
+      if(row < uniforms.dim_inner && col < uniforms.dim_b_outer)
+      {
+        ${getBIndices()}
+        value = ${bVariable.getByIndices('bIndices')};
+      }
+      return value;
+    }
+
+    fn mm_write(batch: i32, row: i32, colIn: i32, valueIn: ${typeSnippet(component, dataType)}) {
+      let col = colIn * ${component};
+      if (row < uniforms.dim_a_outer && col < uniforms.dim_b_outer) {
+        var value = valueIn;
+        let coords = vec3<i32>(batch, row, colIn);
+        ${
+          hasBias ?
+              `value = value + ${isChannelsLast ? 'bias[colIn]' : `${typeSnippet(component, dataType)}(bias[row])`};` :
+                                                  ''                                    }
+        ${applyActivation}
+        ${outputVariable.setByIndices('vec3<u32>(coords)', 'value')}
+      }
+    }
+    `;
+      return source;
+    };
+
+export const createMatmulProgramInfo =
+    (inputs: readonly TensorView[], activationAttributes: InternalActivationAttributes, outputShape: readonly number[],
+     reshapedOutputShape?: readonly number[],
+     isChannelsLast = false /* only used for conv2dByMatMul*/): ProgramInfo => {
+      const aShape = inputs[0].dims;
+      const bShape = inputs[1].dims;
+      const outerDimsA = aShape.slice(0, -2);
+      const outerDimsB = bShape.slice(0, -2);
+      const outerDims = reshapedOutputShape ? reshapedOutputShape.slice(0, -2) : outputShape.slice(0, -2);
+      const batchSize = ShapeUtil.size(outerDims);
+      const dimAOuter = aShape[aShape.length - 2];
+      const dimInner = aShape[aShape.length - 1];
+      const dimBOuter = bShape[bShape.length - 1];
+      const isVec4 = dimInner % 4 === 0 && dimBOuter % 4 === 0;
+
+      // TODO: fine tune size
+      const elementsPerThread = dimAOuter <= 8 ? [4, 1, 1] : [4, 4, 1];
+      const workgroupSize: [number, number, number] = [8, 8, 1];
+      const dispatch = [
+        Math.ceil(dimBOuter / workgroupSize[0] / elementsPerThread[0]),
+        Math.ceil(dimAOuter / workgroupSize[1] / elementsPerThread[1]),
+        Math.ceil(batchSize / workgroupSize[2] / elementsPerThread[2])
+      ];
+
+      const components = isVec4 ? 4 : 1;
+      const aShapeTemp = [...outerDimsA, dimAOuter, dimInner / components];
+      const aRank = aShapeTemp.length;
+      const bShapeTemp = [...outerDimsB, dimInner, dimBOuter / components];
+      const bRank = bShapeTemp.length;
+      const outputShapeTemp = [batchSize, dimAOuter, dimBOuter / components];
+      const programUniforms: ProgramUniform[] = [
+        {type: DataType.int32, data: dimAOuter}, {type: DataType.int32, data: dimBOuter},
+        {type: DataType.int32, data: dimInner}
+      ];
+      appendActivationUniformsData(activationAttributes, programUniforms);
+      programUniforms.push(...createTensorShapeVariables(outerDims, aShapeTemp, bShapeTemp));
+      const inputDependencies: ProgramInputTensorInfoDependency[] = ['rank', 'rank'];
+
+      const hasBias = inputs.length > 2;
+      if (hasBias) {
+        programUniforms.push(...createTensorShapeVariables(inputs[2].dims));
+        inputDependencies.push('rank');
+      }
+      programUniforms.push(...createTensorShapeVariables(outputShapeTemp));
+
+      const getShaderSource = (shaderHelper: ShaderHelper) => {
+        const batchRank = outerDims.length;
+        const batchDims = internalVariable('batchDims', inputs[0].dataType, batchRank, 1);
+        const dataType = tensorTypeToWsglStorageType(inputs[0].dataType);
+
+        const A = inputVariable('a', inputs[0].dataType, aRank, components);
+        const B = inputVariable('b', inputs[1].dataType, bRank, components);
+        const output = outputVariable('result', inputs[0].dataType, outputShapeTemp.length, components);
+        const inputVariables = [A, B];
+        if (hasBias) {
+          const biasComponents = isChannelsLast ? components : 1;
+          inputVariables.push(inputVariable('bias', inputs[2].dataType, inputs[2].dims.length, biasComponents));
+        }
+        const uniforms: UniformsArrayType =
+            [{name: 'dim_a_outer', type: 'i32'}, {name: 'dim_b_outer', type: 'i32'}, {name: 'dim_inner', type: 'i32'}];
+        appendActivationUniforms(activationAttributes, uniforms);
+        const baseType = tensorTypeToWsglStorageType(output.type.tensor);
+        const applyActivation = getActivationSnippet(activationAttributes, output.type.value, baseType);
+        const declareFunctions = matMulReadWriteFnSource(
+            components, hasBias, applyActivation, [batchDims, A, B, output], [outerDimsA, outerDimsB, outerDims],
+            isChannelsLast);
+        return `
+  ${
+            shaderHelper.registerUniforms(uniforms).registerInternalVariables(batchDims).declareVariables(
+                ...inputVariables, output)}
+  ${declareFunctions}
+  ${
+            isVec4 ? makeMatMulPackedVec4Source(elementsPerThread, workgroupSize, dataType, batchDims) :
+                     makeMatMulPackedSource(elementsPerThread, workgroupSize, dataType, batchDims)}
+                   `;
+      };
+      return {
+        name: 'MatMul',
+        shaderCache: {
+          hint: `${elementsPerThread};${activationAttributes.activation};${isVec4};${isChannelsLast}`,
+          inputDependencies
+        },
+        getRunData: () => ({
+          outputs: [{dims: outputShape, dataType: inputs[0].dataType}],
+          dispatchGroup: {x: dispatch[0], y: dispatch[1], z: dispatch[2]},
+          programUniforms
+        }),
+        getShaderSource,
+      };
+    };
