@@ -2,8 +2,7 @@
 // Licensed under the MIT License.
 
 import * as base64 from 'base64-js';
-import * as fs from 'fs';
-import {promisify} from 'util';
+import * as fs from 'node:fs/promises';
 
 import {Attribute} from '../lib/onnxjs/attribute';
 import {Graph} from '../lib/onnxjs/graph';
@@ -16,14 +15,33 @@ export function bufferToBase64(buffer: Uint8Array): string {
   return base64.fromByteArray(buffer);
 }
 
+async function retry<T>(fn: () => Promise<T>, maxRetries = 3, delay = 100): Promise<T> {
+  let retries = maxRetries;
+  do {
+    try {
+      return await fn();
+    } catch (err) {
+      if (retries-- === 0) {
+        throw err;
+      }
+      await new Promise(resolve => setTimeout(resolve, delay));
+    }
+    // eslint-disable-next-line no-constant-condition
+  } while (true);
+}
+
 export async function readFile(file: string) {
   if (typeof process !== 'undefined' && process.versions && process.versions.node) {
     // node
-    return promisify(fs.readFile)(file);
+    return fs.readFile(file);
   } else {
     // browser
-    const response = await fetch(file);
-    return new Uint8Array(await response.arrayBuffer());
+    //
+    // use "retry" to workaround the error "TypeError: Failed to fetch" in some test environments
+    return retry(async () => {
+      const response = await fetch(file);
+      return new Uint8Array(await response.arrayBuffer());
+    });
   }
 }
 
