@@ -10,6 +10,10 @@ namespace onnxruntime {
 namespace js {
 
 void* JsCustomAllocator::Alloc(size_t size) {
+  if (size == 0) {
+    return nullptr;
+  }
+
   void* p = EM_ASM_PTR({ return Module.jsepAlloc($0); }, size);
   stats_.num_allocs++;
   stats_.bytes_in_use += size;
@@ -17,8 +21,10 @@ void* JsCustomAllocator::Alloc(size_t size) {
 }
 
 void JsCustomAllocator::Free(void* p) {
-  size_t size = (size_t)(void*)EM_ASM_PTR({ return Module.jsepFree($0); }, p);
-  stats_.bytes_in_use -= size;
+  if (p != nullptr) {
+    size_t size = (size_t)(void*)EM_ASM_PTR({ return Module.jsepFree($0); }, p);
+    stats_.bytes_in_use -= size;
+  }
 }
 
 void JsCustomAllocator::GetStats(AllocatorStats* stats) {
