@@ -11,9 +11,11 @@
 struct OrtOpenVINOProviderOptions;
 
 namespace onnxruntime {
+struct SessionOptions;
 // defined in provider_bridge_ort.cc
 struct OpenVINOProviderFactoryCreator {
-  static std::shared_ptr<IExecutionProviderFactory> Create(const ProviderOptions* provider_options_map);
+  static std::shared_ptr<IExecutionProviderFactory> Create(ProviderOptions* provider_options_map,
+                                                           const SessionOptions* session_options);
   static std::shared_ptr<IExecutionProviderFactory> Create(const OrtOpenVINOProviderOptions* provider_options);
 };
 }  // namespace onnxruntime
