@@ -4,7 +4,7 @@
 #pragma once
 
 #include "core/graph/graph_viewer.h"
-#include "core/providers/shared/node_unit/node_unit.h"
+#include "core/framework/node_unit.h"
 #include "core/providers/shared/utils/utils.h"
 
 namespace onnxruntime {
@@ -19,14 +19,12 @@ class IOpBuilder {
   // Check whether the operator is supported or not
   virtual Status IsOpSupported(QnnModelWrapper& qnn_model_wrapper,
                                const NodeUnit& node,
-                               const logging::Logger& logger,
-                               bool is_quantized_model = true) const ORT_MUST_USE_RESULT = 0;
+                               const logging::Logger& logger) const ORT_MUST_USE_RESULT = 0;
 
   // Add the operator to QNN model
   virtual Status AddToModelBuilder(QnnModelWrapper& qnn_model_wrapper,
                                    const NodeUnit& node,
                                    const logging::Logger& logger,
-                                   bool is_quantized_model = true,
                                    bool do_op_validation = false) const ORT_MUST_USE_RESULT = 0;
 
   virtual std::string GetOpBuilderType() const = 0;
