@@ -6,15 +6,15 @@
 #include <vector>
 
 #include "core/common/common.h"
+#include "core/common/safeint.h"
+#include "core/framework/node_unit.h"
 #include "core/framework/tensorprotoutils.h"
 #include "core/graph/indexed_sub_graph.h"
 #include "core/graph/node_attr_utils.h"
-
-#include "core/providers/shared/node_unit/node_unit.h"
-#include "onnx/defs/attr_proto_util.h"
-#include "core/common/safeint.h"
 #include "core/optimizer/initializer.h"
 
+#include "onnx/defs/attr_proto_util.h"
+
 namespace onnxruntime {
 namespace xnnpack {
 
@@ -25,7 +25,7 @@ const char* OpTypeToString(OpComputeType opCtype) {
     case op_compute_type_fp16:
       return "fp16";
     case op_compute_type_qs8_per_channel:
-      return "qc8";
+      return "qs8_qc8w";
     case op_compute_type_qs8:
       return "qs8";
     case op_compute_type_qu8:
