@@ -394,8 +394,8 @@ static void RunMultiHeadAttentionTests(AttentionTestData& data, bool disable_cpu
     }
 
 #if USE_MEMORY_EFFICIENT_ATTENTION
-    if (data.sequence_length >= contrib::attention::kMinSeqLenForMemoryEfficientAttentionFp32 ||
-        data.kv_sequence_length >= contrib::attention::kMinSeqLenForMemoryEfficientAttentionFp32) {
+    if (data.sequence_length >= contrib::attention::kDefaultMinSeqLenForEfficientAttentionFp32 ||
+        data.kv_sequence_length >= contrib::attention::kDefaultMinSeqLenForEfficientAttentionFp32) {
       kernel_type = AttentionKernelType::AttentionKernel_CutlassMemoryEfficientAttention;
       if (!SkipAttentionKernel(data, kernel_type)) {
         RunMultiHeadAttentionKernel(
@@ -483,7 +483,6 @@ TEST(MultiHeadAttentionTest, CrossAttention_Batch2_HeadSize32_RightSidePadding_M
 }
 
 TEST(MultiHeadAttentionTest, CrossAttention_Batch2_HeadSize32_RightSidePadding_Mask2D) {
-  ROCM_GTEST_SKIP("ROCm MHA expect failure due to ck bug");
   AttentionTestData data;
   GetCrossAttentionData_Batch2_HeadSize32_RightSidePadding(data, false);
   RunMultiHeadAttentionTests(data, true);
@@ -532,12 +531,22 @@ TEST(MultiHeadAttentionTest, CrossAttention_Batch1_HeadSize16) {
   RunMultiHeadAttentionTests(data);
 }
 
+TEST(MultiHeadAttentionTest, CrossAttention_Batch1_HeadSize8) {
+  AttentionTestData data;
+  GetCrossAttentionData_HeadSize8_NoBias(data);
+  RunMultiHeadAttentionTests(data, false, true);
+}
+
+// TODO (pavignol): Fix this regression
+// Bug #50220930
+#ifndef USE_DML
 TEST(MultiHeadAttentionTest, CrossAttentionWithPast) {
   ROCM_GTEST_SKIP("ROCm MHA only support head_size >= 8");
   AttentionTestData data;
   GetCrossAttentionDataWithPast(data);
   RunMultiHeadAttentionTests(data);
 }
+#endif
 
 TEST(MultiHeadAttentionTest, SelfAttention_WithPast_WithRelPosBias_ForT5) {
   ROCM_GTEST_SKIP("ROCm MHA only support head_size >= 8");
