@@ -27,7 +27,7 @@ void TestConvTransposeOpInitializer(const ConvTransposeOpAttributes& attributes,
                                     const vector<vector<int64_t>>& input_shapes,
                                     const std::initializer_list<float>& expected_output,
                                     const vector<int64_t>& expected_output_shape,
-                                    bool is_filter_initializer = false,
+                                    bool is_weight_and_bias_initializer = false,
                                     OpTester::ExpectResult expect_result = OpTester::ExpectResult::kExpectSuccess,
                                     const std::string& err_str = "",
                                     const std::unordered_set<std::string>& excluded_provider_types = {kTensorrtExecutionProvider}) {
@@ -58,10 +58,10 @@ void TestConvTransposeOpInitializer(const ConvTransposeOpAttributes& attributes,
   }
 
   ORT_ENFORCE(inputs.size() <= 3, "Our name array is only setup to handle 3 inputs");
-  const char* szNames[] = {"X", "W", "B"};
-  bool isInitializers[] = {false, is_filter_initializer, false};
+  const char* input_names[] = {"X", "W", "B"};
+  bool is_initializers[] = {false, is_weight_and_bias_initializer, is_weight_and_bias_initializer};
   for (size_t i = 0; i < inputs.size(); i++) {
-    test.AddInput<float>(szNames[i], input_shapes[i], inputs[i], isInitializers[i]);
+    test.AddInput<float>(input_names[i], input_shapes[i], inputs[i], is_initializers[i]);
   }
   test.AddOutput<float>("Y", expected_output_shape, expected_output);
 
@@ -75,7 +75,8 @@ void TestConvTransposeOp(const ConvTransposeOpAttributes& attributes,
                          const vector<int64_t>& expected_output_shape,
                          OpTester::ExpectResult expect_result = OpTester::ExpectResult::kExpectSuccess,
                          const std::string& err_str = "",
-                         const std::unordered_set<std::string>& excluded_provider_types = {kTensorrtExecutionProvider, kQnnExecutionProvider}) {
+                         const std::unordered_set<std::string>& excluded_provider_types =
+                             {kCudaNHWCExecutionProvider, kTensorrtExecutionProvider, kQnnExecutionProvider}) {
   std::unordered_set<std::string> extra_exclude_openvino_for_initializer_filter = excluded_provider_types;
   extra_exclude_openvino_for_initializer_filter.insert(kOpenVINOExecutionProvider);
   TestConvTransposeOpInitializer(attributes, inputs, input_shapes, expected_output, expected_output_shape,
@@ -97,17 +98,23 @@ TEST(ConvTransposeTest, ConvTranspose_1D) {
       1,                      // group
       "NOTSET"                // auto_pad
   };
-  vector<float> X = {0.0f, 1.0f, 2.0f};
-  vector<int64_t> X_shape = {1, 1, 3};
-  vector<float> W = {1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f};
-  vector<int64_t> W_shape = {1, 2, 3};
+
+  vector<int64_t> X_shape = {1, 2, 3};
+  vector<float> X = {0.1f, 1.0f, 2.0f,
+                     3.0f, 4.0f, 5.0f};
+  vector<int64_t> W_shape = {2, 2, 3};
+  vector<float> W = {1.0f, 2.0f, 3.0f,
+                     4.0f, 5.0f, 6.0f,
+                     6.0f, 5.0f, 4.0f,
+                     3.0f, 2.0f, 1.0f};
   vector<int64_t> Y_shape = {1, 2, 5};
-  auto expected_vals = {0.0f, 1.0f, 3.0f, 3.0f, 2.0f, 0.0f, 1.0f, 3.0f, 3.0f, 2.0f};
+  auto expected_vals = {18.1f, 40.2f, 66.3f, 48.f, 26.f,
+                        9.4f, 22.5f, 39.6f, 30.f, 17.f};
 
   TestConvTransposeOp(attrs, {X, W}, {X_shape, W_shape}, expected_vals, Y_shape);
 }
 
-TEST(ConvTransposeTest, ConvTranspose_2D) {
+TEST(ConvTransposeTest, ConvTranspose_2D_outputpadding_strides2) {
   ConvTransposeOpAttributes attrs = {
       vector<int64_t>{3, 3},        // kernel_shape
       vector<int64_t>{1, 1},        // output_padding
@@ -118,14 +125,17 @@ TEST(ConvTransposeTest, ConvTranspose_2D) {
       1,                            // group
       "NOTSET"                      // auto_pad
   };
+
+  vector<int64_t> X_shape = {1, 1, 3, 3};
   vector<float> X = {0.16857791f, -0.15161794f, 0.08540368f,
                      0.1820628f, -0.21746576f, 0.08245695f,
                      0.1431433f, -0.43156421f, 0.30591947f};
-  vector<int64_t> X_shape = {1, 1, 3, 3};
+
+  vector<int64_t> W_shape = {1, 1, 3, 3};
   vector<float> W = {-0.06230065f, 0.37932432f, -0.25388849f,
                      0.33878803f, 0.43709868f, -0.22477469f,
                      0.04118127f, -0.44696793f, 0.06373066f};
-  vector<int64_t> W_shape = {1, 1, 3, 3};
+
   vector<int64_t> Y_shape = {1, 1, 6, 6};
   auto expected_vals = {0.07368518f, -0.08925839f, -0.06627201f, 0.06301362f, 0.03732984f, -0.01919658f,
                         -0.00628807f, -0.02817563f, -0.01472169f, 0.04392925f, -0.00689478f, -0.01549204f,
@@ -136,6 +146,45 @@ TEST(ConvTransposeTest, ConvTranspose_2D) {
   TestConvTransposeOp(attrs, {X, W}, {X_shape, W_shape}, expected_vals, Y_shape);
 }
 
+// 2D input with C > 1
+TEST(ConvTransposeTest, ConvTranspose_2D_C2) {
+  ConvTransposeOpAttributes attrs = {
+      vector<int64_t>{2, 2},        // kernel_shape
+      {},                           // output_padding
+      {},                           // output_shape
+      vector<int64_t>{0, 0, 0, 0},  // pads
+      vector<int64_t>{1, 1},        // strides
+      vector<int64_t>{1, 1},        // dilations
+      1,                            // group
+      "NOTSET"                      // auto_pad
+  };
+
+  vector<int64_t> X_shape = {1, 2, 3, 3};
+  vector<float> X = {0.43f, 0.42871707f, 0.29552766f,
+                     0.17258859f, 0.68087016f, 0.7090254f,
+                     0.60937387f, 0.58646585f, 0.84525721f,
+
+                     0.47011843f, 0.95854213f, 0.3972888f,
+                     0.0585452f, 0.1206734f, 0.76727852f,
+                     0.46040912f, 0.83495316f, 0.02409773f};
+
+  vector<int64_t> W_shape = {2, 1, 2, 2};
+  vector<float> W = {0.25616416f, 0.10246604f,
+                     0.08771133f, 0.30770606f,
+
+                     0.84369617f, 0.3010619f,
+                     0.44524362f, 0.6056068f};
+
+  vector<int64_t> Y_shape = {1, 1, 4, 4};
+  auto expected_vals = {
+      0.50678771f, 1.10413539f, 0.74340409f, 0.14989006f,
+      0.34063845f, 1.19294512f, 1.85030293f, 0.63518577f,
+      0.58575004f, 1.25774109f, 1.23472511f, 0.77670550f,
+      0.25844323f, 0.88953220f, 0.77098041f, 0.27468451f};
+
+  TestConvTransposeOp(attrs, {X, W}, {X_shape, W_shape}, expected_vals, Y_shape);
+}
+
 TEST(ConvTransposeTest, ConvTranspose_2D_Bias_1) {
   ConvTransposeOpAttributes attrs = {
       vector<int64_t>{3, 3},        // kernel_shape
@@ -260,7 +309,48 @@ TEST(ConvTransposeTest, ConvTranspose_2D_OutputShape_1) {
                       {kTensorrtExecutionProvider, kOpenVINOExecutionProvider, kQnnExecutionProvider});
 }
 
-TEST(ConvTransposeTest, ConvTranspose_2D_OutputShape_1_group_2_for_tranpose_path) {
+TEST(ConvTransposeTest, ConvTranspose_1D_OutputShape_1_group_2_for_transpose_path) {
+  ConvTransposeOpAttributes attrs = {
+      vector<int64_t>{3},        // kernel_shape
+      {},                        // output_padding
+      vector<int64_t>{1, 6, 4},  // output_shape
+      vector<int64_t>{0, 0},     // pads
+      vector<int64_t>{1},        // strides
+      vector<int64_t>{1},        // dilations
+      2,                         // group
+      "NOTSET"                   // auto_pad
+  };
+  int image_size = 4;
+  int input_channels = 3 * 2;
+  int output_channels = 3;
+  std::vector<float> X;
+  for (int i = 0; i < input_channels * image_size; i++) {
+    X.push_back(1.0f);
+  }
+
+  std::vector<float> W;
+  int kernel_size = output_channels * input_channels * 3;
+  for (int i = 0; i < kernel_size; i++) {
+    W.push_back(1.0f);
+  }
+
+  vector<int64_t> X_shape = {1, 6, 4};
+  vector<int64_t> W_shape = {6, 3, 3};
+  vector<int64_t> Y_shape = {1, 6, 4};
+
+  auto expected_vals = {6.0f, 9.0f, 9.0f, 6.0f,
+                        6.0f, 9.0f, 9.0f, 6.0f,
+                        6.0f, 9.0f, 9.0f, 6.0f,
+                        6.0f, 9.0f, 9.0f, 6.0f,
+                        6.0f, 9.0f, 9.0f, 6.0f,
+                        6.0f, 9.0f, 9.0f, 6.0f};
+
+  TestConvTransposeOp(attrs, {X, W}, {X_shape, W_shape}, expected_vals, Y_shape,
+                      OpTester::ExpectResult::kExpectSuccess, "",
+                      {kTensorrtExecutionProvider, kOpenVINOExecutionProvider, kQnnExecutionProvider});
+}
+
+TEST(ConvTransposeTest, ConvTranspose_2D_OutputShape_1_group_2_for_transpose_path) {
   ConvTransposeOpAttributes attrs = {
       vector<int64_t>{3, 3},        // kernel_shape
       {},                           // output_padding
@@ -286,104 +376,31 @@ TEST(ConvTransposeTest, ConvTranspose_2D_OutputShape_1_group_2_for_tranpose_path
   vector<int64_t> W_shape = {6, 3, 3, 3};
 
   vector<int64_t> Y_shape = {1, 6, 4, 4};
-  auto expected_vals = {
-      12.0f,
-      18.0f,
-      18.0f,
-      12.0f,
-      18.0f,
-      27.0f,
-      27.0f,
-      18.0f,
-      18.0f,
-      27.0f,
-      27.0f,
-      18.0f,
-      12.0f,
-      18.0f,
-      18.0f,
-      12.0f,
-      12.0f,
-      18.0f,
-      18.0f,
-      12.0f,
-      18.0f,
-      27.0f,
-      27.0f,
-      18.0f,
-      18.0f,
-      27.0f,
-      27.0f,
-      18.0f,
-      12.0f,
-      18.0f,
-      18.0f,
-      12.0f,
-      12.0f,
-      18.0f,
-      18.0f,
-      12.0f,
-      18.0f,
-      27.0f,
-      27.0f,
-      18.0f,
-      18.0f,
-      27.0f,
-      27.0f,
-      18.0f,
-      12.0f,
-      18.0f,
-      18.0f,
-      12.0f,  // duplicate below
-      12.0f,
-      18.0f,
-      18.0f,
-      12.0f,
-      18.0f,
-      27.0f,
-      27.0f,
-      18.0f,
-      18.0f,
-      27.0f,
-      27.0f,
-      18.0f,
-      12.0f,
-      18.0f,
-      18.0f,
-      12.0f,
-      12.0f,
-      18.0f,
-      18.0f,
-      12.0f,
-      18.0f,
-      27.0f,
-      27.0f,
-      18.0f,
-      18.0f,
-      27.0f,
-      27.0f,
-      18.0f,
-      12.0f,
-      18.0f,
-      18.0f,
-      12.0f,
-      12.0f,
-      18.0f,
-      18.0f,
-      12.0f,
-      18.0f,
-      27.0f,
-      27.0f,
-      18.0f,
-      18.0f,
-      27.0f,
-      27.0f,
-      18.0f,
-      12.0f,
-      18.0f,
-      18.0f,
-      12.0f,
-  };
+  auto expected_vals = {12.0f, 18.0f, 18.0f, 12.0f,
+                        18.0f, 27.0f, 27.0f, 18.0f,
+                        18.0f, 27.0f, 27.0f, 18.0f,
+                        12.0f, 18.0f, 18.0f, 12.0f,
+                        12.0f, 18.0f, 18.0f, 12.0f,
+                        18.0f, 27.0f, 27.0f, 18.0f,
+                        18.0f, 27.0f, 27.0f, 18.0f,
+                        12.0f, 18.0f, 18.0f, 12.0f,
+                        12.0f, 18.0f, 18.0f, 12.0f,
+                        18.0f, 27.0f, 27.0f, 18.0f,
+                        18.0f, 27.0f, 27.0f, 18.0f,
+                        12.0f, 18.0f, 18.0f, 12.0f,  // duplicate below
+                        12.0f, 18.0f, 18.0f, 12.0f,
+                        18.0f, 27.0f, 27.0f, 18.0f,
+                        18.0f, 27.0f, 27.0f, 18.0f,
+                        12.0f, 18.0f, 18.0f, 12.0f,
+                        12.0f, 18.0f, 18.0f, 12.0f,
+                        18.0f, 27.0f, 27.0f, 18.0f,
+                        18.0f, 27.0f, 27.0f, 18.0f,
+                        12.0f, 18.0f, 18.0f, 12.0f,
+                        12.0f, 18.0f, 18.0f, 12.0f,
+                        18.0f, 27.0f, 27.0f, 18.0f,
+                        18.0f, 27.0f, 27.0f, 18.0f,
+                        12.0f, 18.0f, 18.0f, 12.0f};
+
   TestConvTransposeOp(attrs, {X, W}, {X_shape, W_shape}, expected_vals, Y_shape,
                       OpTester::ExpectResult::kExpectSuccess, "",
                       {kTensorrtExecutionProvider, kOpenVINOExecutionProvider, kQnnExecutionProvider});
@@ -409,7 +426,8 @@ TEST(ConvTransposeTest, ConvTranspose_2D_OutputShape_2) {
   vector<int64_t> Y_shape = {1, 1, 1, 14};
   auto expected_vals = {1.0f, 2.0f, 5.0f, 11.0f, 19.0f, 28.0f, 37.0f, 46.0f, 55.0f, 64.0f, 63.0f, 51.0f, 27.0f, 10.0f};
   TestConvTransposeOp(attrs, {X, W, B}, {X_shape, W_shape, B_shape}, expected_vals, Y_shape,
-                      OpTester::ExpectResult::kExpectSuccess, "", {kOpenVINOExecutionProvider, kQnnExecutionProvider});
+                      OpTester::ExpectResult::kExpectSuccess, "",
+                      {kOpenVINOExecutionProvider, kCudaNHWCExecutionProvider, kQnnExecutionProvider});
 }
 
 TEST(ConvTransposeTest, ConvTranspose_2D_OutputShapeWithBatchSize) {
@@ -434,15 +452,11 @@ TEST(ConvTransposeTest, ConvTranspose_2D_OutputShapeWithBatchSize) {
   auto expected_vals = {1.0f, 2.0f, 5.0f, 11.0f, 19.0f, 28.0f, 37.0f, 46.0f, 55.0f, 64.0f, 63.0f, 51.0f, 27.0f, 10.0f,
                         11.0f, 32.0f, 65.0f, 91.0f, 109.0f, 118.0f, 127.0f, 136.0f, 145.0f, 154.0f, 143.0f, 111.0f, 57.0f, 20.0f};
   TestConvTransposeOp(attrs, {X, W, B}, {X_shape, W_shape, B_shape}, expected_vals, Y_shape,
-                      OpTester::ExpectResult::kExpectSuccess, "", {kOpenVINOExecutionProvider, kQnnExecutionProvider});
+                      OpTester::ExpectResult::kExpectSuccess, "",
+                      {kOpenVINOExecutionProvider, kCudaNHWCExecutionProvider, kQnnExecutionProvider});
 }
 
 TEST(ConvTransposeTest, ConvTranspose_InvalidKernelShape) {
-  // TODO: Unskip when fixed #41968513
-  if (DefaultDmlExecutionProvider().get() != nullptr) {
-    GTEST_SKIP() << "Skipping because of the following error: provider_test_utils.cc(866): error: Value of: expect_result == ExpectResult::kExpectSuccess";
-  }
-
   ConvTransposeOpAttributes attrs = {
       vector<int64_t>{1, 1, 1, 5},   // invalid kernel_shape, should be [1, 5]
       {},                            // output_padding
@@ -465,7 +479,11 @@ TEST(ConvTransposeTest, ConvTranspose_InvalidKernelShape) {
                         11.0f, 32.0f, 65.0f, 91.0f, 109.0f, 118.0f, 127.0f, 136.0f, 145.0f, 154.0f, 143.0f, 111.0f, 57.0f, 20.0f};
   TestConvTransposeOp(attrs, {X, W, B}, {X_shape, W_shape, B_shape}, expected_vals, Y_shape,
                       OpTester::ExpectResult::kExpectFailure,
-                      "kernel_shape num_dims is not compatible with W num_dims. kernel_shape: {1,1,1,5} W: {1,1,1,5}");
+                      // error message will end in "W: {1,1,1,5}" or "W: {1,1,5,1} depending on whether NCHW or NHWC,
+                      // so drop the part that differs from the expected string
+                      "kernel_shape num_dims is not compatible with W num_dims. kernel_shape: {1,1,1,5} W: {1,1,",
+                      {kTensorrtExecutionProvider, kQnnExecutionProvider,
+                       kDmlExecutionProvider});  // TODO: Unskip when fixed #41968513
 }
 
 TEST(ConvTransposeTest, ConvTranspose_onnx) {
@@ -871,7 +889,8 @@ TEST(ConvTransposeTest, DimWithZero) {
 
   TestConvTransposeOp(attrs, {X, W}, {X_shape, W_shape}, expected_vals, Y_shape,
                       OpTester::ExpectResult::kExpectSuccess, "",
-                      {kTensorrtExecutionProvider, kAclExecutionProvider, kQnnExecutionProvider});
+                      {kCudaNHWCExecutionProvider, kTensorrtExecutionProvider,
+                       kAclExecutionProvider, kQnnExecutionProvider});
 }
 
 TEST(ConvTransposeTest, ConvTranspose_3D) {
@@ -1005,7 +1024,8 @@ TEST(ConvTransposeTest, ConvTranspose_3D) {
 
   TestConvTransposeOp(attrs, {X, W, B}, {X_shape, W_shape, B_shape}, expected_vals, Y_shape,
                       OpTester::ExpectResult::kExpectSuccess, "",
-                      {kTensorrtExecutionProvider, kCudaExecutionProvider, kQnnExecutionProvider});
+                      {kTensorrtExecutionProvider, kCudaExecutionProvider,
+                       kCudaNHWCExecutionProvider, kQnnExecutionProvider});
 }
 
 TEST(ConvTransposeTest, ConvTranspose_1D_AsymmetricPads) {
@@ -1080,11 +1100,6 @@ TEST(ConvTransposeTest, ConvTranspose_1D_AutoPad_SameLower) {
 }
 
 TEST(ConvTransposeTest, ConvTranspose_AutoPad_with_non_default_strides) {
-  // TODO: Unskip when fixed #41968513
-  if (DefaultDmlExecutionProvider().get() != nullptr) {
-    GTEST_SKIP() << "Skipping because of the following error: MLOperatorAuthorImpl.cpp(2100): The parameter is incorrect.";
-  }
-
   ConvTransposeOpAttributes attrs = {
       vector<int64_t>{3, 3},  // kernel_shape
       {},                     // output_padding
@@ -1127,7 +1142,9 @@ TEST(ConvTransposeTest, ConvTranspose_AutoPad_with_non_default_strides) {
 
   TestConvTransposeOp(attrs, {X, W}, {X_shape, W_shape}, expected_vals, Y_shape,
                       OpTester::ExpectResult::kExpectSuccess, "",
-                      {kTensorrtExecutionProvider, kOpenVINOExecutionProvider, kQnnExecutionProvider});  // Accuracy Mismatch on OpenVINO-EP
+                      {kTensorrtExecutionProvider, kQnnExecutionProvider,
+                       kOpenVINOExecutionProvider,  // Accuracy Mismatch on OpenVINO-EP
+                       kDmlExecutionProvider});     // TODO: Unskip when fixed #41968513
 }
 
 #ifndef ENABLE_TRAINING
