@@ -37,7 +37,7 @@ TEST(TransposeOpTest, PermRankDoesNotMatchTensorRank) {
   // This failure comes from shape inference, because in this case it knows the input dims.
   // But in the real world, the model can supply different input dims at runtime.
   test.Run(OpTester::ExpectResult::kExpectFailure,
-           "Node:node1 Output:Y [ShapeInferenceError] Mismatch between number of source and target dimensions. Source=3 Target=4");
+           "Node:node1 Output:Y [ShapeInferenceError] Mismatch between number of inferred and declared dimensions. inferred=3 declared=4");
 }
 
 // Some of the tests can't run on TensorrtExecutionProvider because of errors.
@@ -49,14 +49,17 @@ void TransposeTest(const std::vector<int64_t>& input_shape,
                    const std::vector<int64_t>* p_perm,
                    const std::vector<int64_t>& expected_shape,
                    const std::vector<T>& expected_vals,
-                   const std::unordered_set<std::string>& excluded_provider_types = {}) {
-  OpTester test("Transpose");
-  if (nullptr != p_perm)
-    test.AddAttribute("perm", *p_perm);
-  test.AddInput<T>("X", input_shape, input_vals);
-  test.AddOutput<T>("Y", expected_shape, expected_vals);
+                   const std::unordered_set<std::string>& excluded_provider_types = {},
+                   const std::vector<int>& opsets = {7}) {
+  for (auto opset : opsets) {
+    OpTester test("Transpose", opset);
+    if (nullptr != p_perm)
+      test.AddAttribute("perm", *p_perm);
+    test.AddInput<T>("X", input_shape, input_vals);
+    test.AddOutput<T>("Y", expected_shape, expected_vals);
 
-  test.Run(OpTester::ExpectResult::kExpectSuccess, "", excluded_provider_types);
+    test.Run(OpTester::ExpectResult::kExpectSuccess, "", excluded_provider_types);
+  }
 }
 
 // Test 2 dimensional transpose, with no permutation attribute specified
@@ -73,7 +76,7 @@ TEST(TransposeOpTest, TwoDimNoAttr) {
       3.0f, 6.0f};
 
   TransposeTest(input_shape, input_vals, nullptr, expected_shape, expected_vals,
-                {kTensorrtExecutionProvider});  // TensorRT: SegFault error
+                {kTensorrtExecutionProvider}, {7, 21});  // TensorRT: SegFault error
 }
 
 TEST(TransposeOpTest, TwoDimNoAttrStr) {
@@ -88,7 +91,7 @@ TEST(TransposeOpTest, TwoDimNoAttrStr) {
       "2", "5",
       "3", "6"};
 
-  TransposeTest(input_shape, input_vals, nullptr, expected_shape, expected_vals);
+  TransposeTest(input_shape, input_vals, nullptr, expected_shape, expected_vals, {}, {7, 21});
 }
 
 // Test 2 dimensional transpose, with permutation attribute specified
@@ -103,7 +106,47 @@ TEST(TransposeOpTest, TwoDim) {
                                       2.0f, 5.0f,
                                       3.0f, 6.0f};
 
-  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals);
+  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals, {}, {7, 21});
+}
+
+// Test Int4 transpose with odd inner dimension.
+TEST(TransposeOpTest, TwoDim_Odd_Int4) {
+  constexpr int8_t unused_val = 0;
+  std::vector<int64_t> input_shape({5, 3});
+  std::vector<Int4x2> input_vals = {Int4x2(1, 2), Int4x2(3, 4), Int4x2(5, 6), Int4x2(7, 8),
+                                    Int4x2(9, 10), Int4x2(11, 12), Int4x2(13, 14), Int4x2(15, unused_val)};
+
+  std::vector<int64_t> perm = {1, 0};
+  std::vector<int64_t> expected_shape({3, 5});
+  std::vector<Int4x2> expected_vals = {Int4x2(1, 4), Int4x2(7, 10), Int4x2(13, 2), Int4x2(5, 8),
+                                       Int4x2(11, 14), Int4x2(3, 6), Int4x2(9, 12), Int4x2(15, unused_val)};
+
+  OpTester test("Transpose", 21);
+  test.AddAttribute("perm", perm);
+  test.AddInput<Int4x2>("X", input_shape, input_vals);
+  test.AddOutput<Int4x2>("Y", expected_shape, expected_vals);
+
+  test.Run(OpTester::ExpectResult::kExpectSuccess, "", {kTensorrtExecutionProvider});
+}
+
+// Test UInt4 transpose with odd inner dimension.
+TEST(TransposeOpTest, TwoDim_Odd_UInt4) {
+  constexpr int8_t unused_val = 0;
+  std::vector<int64_t> input_shape({5, 3});
+  std::vector<UInt4x2> input_vals = {UInt4x2(1, 2), UInt4x2(3, 4), UInt4x2(5, 6), UInt4x2(7, 8),
+                                     UInt4x2(9, 10), UInt4x2(11, 12), UInt4x2(13, 14), UInt4x2(15, unused_val)};
+
+  std::vector<int64_t> perm = {1, 0};
+  std::vector<int64_t> expected_shape({3, 5});
+  std::vector<UInt4x2> expected_vals = {UInt4x2(1, 4), UInt4x2(7, 10), UInt4x2(13, 2), UInt4x2(5, 8),
+                                        UInt4x2(11, 14), UInt4x2(3, 6), UInt4x2(9, 12), UInt4x2(15, unused_val)};
+
+  OpTester test("Transpose", 21);
+  test.AddAttribute("perm", perm);
+  test.AddInput<UInt4x2>("X", input_shape, input_vals);
+  test.AddOutput<UInt4x2>("Y", expected_shape, expected_vals);
+
+  test.Run(OpTester::ExpectResult::kExpectSuccess, "", {kTensorrtExecutionProvider});
 }
 
 TEST(TransposeOpTest, TwoDim_double) {
@@ -131,7 +174,7 @@ TEST(TransposeOpTest, TwoDim_int32) {
                                         2, 5,
                                         3, 6};
 
-  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals);
+  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals, {}, {7, 21});
 }
 
 TEST(TransposeOpTest, TwoDim_int16) {
@@ -147,7 +190,7 @@ TEST(TransposeOpTest, TwoDim_int16) {
       2, 5,
       3, 6};
 
-  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals, {kOpenVINOExecutionProvider});
+  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals, {kOpenVINOExecutionProvider}, {7, 21});
 }
 
 TEST(TransposeOpTest, TwoDim_mlfloat16) {
@@ -163,7 +206,7 @@ TEST(TransposeOpTest, TwoDim_mlfloat16) {
        MLFloat16::FromBits(static_cast<uint16_t>(2)), MLFloat16::FromBits(static_cast<uint16_t>(5)),
        MLFloat16::FromBits(static_cast<uint16_t>(3)), MLFloat16::FromBits(static_cast<uint16_t>(6))};
 
-  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals, {kTensorrtExecutionProvider});
+  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals, {kTensorrtExecutionProvider}, {7, 21});
 }
 
 #if defined(USE_DNNL)
@@ -264,7 +307,7 @@ TEST(TransposeOpTest, TwoDim_int8) {
                                        2, 5,
                                        3, 6};
 
-  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals, {kTensorrtExecutionProvider});
+  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals, {kTensorrtExecutionProvider}, {7, 21});
 }
 
 TEST(TransposeOpTest, TwoDimStr) {
@@ -280,7 +323,7 @@ TEST(TransposeOpTest, TwoDimStr) {
       "2", "5",
       "3", "6"};
 
-  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals);
+  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals, {}, {7, 21});
 }
 
 // Test 3 dimensional transpose, with permutation attribute specified
@@ -319,7 +362,7 @@ TEST(TransposeOpTest, Transpose021) {
       3.3f, 6.3f};
 
   TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals,
-                {kTensorrtExecutionProvider});  // TensorRT: illegal error
+                {kTensorrtExecutionProvider}, {7, 21});  // TensorRT: illegal error
 }
 
 TEST(TransposeOpTest, Transpose120) {
@@ -349,7 +392,7 @@ TEST(TransposeOpTest, Transpose120) {
       6.0f, 6.1f, 6.2f, 6.3f};
 
   TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals,
-                {kTensorrtExecutionProvider});  // TensorRT: illegal error
+                {kTensorrtExecutionProvider}, {7, 21});  // TensorRT: illegal error
 }
 
 // test when the suffix size is > 1 (last dimension is not moved)
@@ -382,7 +425,7 @@ TEST(TransposeOpTest, Transpose102) {
       4.3f, 5.3f, 6.3f};
 
   TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals,
-                {kTensorrtExecutionProvider});  // TensorRT: illegal error
+                {kTensorrtExecutionProvider}, {7, 21});  // TensorRT: illegal error
 }
 
 TEST(TransposeOpTest, TransposeReshape) {
@@ -416,7 +459,7 @@ TEST(TransposeOpTest, TransposeReshape) {
       4.3f, 5.3f, 6.3f};
 
   TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals,
-                {kTensorrtExecutionProvider});  // TensorRT: illegal error
+                {kTensorrtExecutionProvider}, {7, 21});  // TensorRT: illegal error
 }
 
 TEST(TransposeOpTest, ThreeDimStr) {
@@ -453,7 +496,7 @@ TEST(TransposeOpTest, ThreeDimStr) {
       "2", "5",
       "3", "6"};
 
-  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals);
+  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals, {}, {7, 21});
 }
 
 TEST(TransposeOpTest, SixDim) {
@@ -478,7 +521,7 @@ TEST(TransposeOpTest, SixDim) {
   }();
 
   TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals,
-                {kQnnExecutionProvider});  // Error: Failed to finalize QNN graph.
+                {kQnnExecutionProvider}, {7, 21});  // Error: Failed to finalize QNN graph.
 }
 
 template <typename T>
@@ -522,7 +565,7 @@ TEST(TransposeOpTest, NCHW2NHWCStr) {
       "3", "7", "11",
       "4", "8", "12"};
 
-  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals, {kTensorrtExecutionProvider});
+  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals, {kTensorrtExecutionProvider}, {7, 21});
 }
 
 template <typename T>
@@ -582,7 +625,7 @@ TEST(TransposeOpTest, NHWC2NCHW_String) {
       "2", "5", "8", "11",
       "3", "6", "9", "12"};
 
-  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals, {kTensorrtExecutionProvider});
+  TransposeTest(input_shape, input_vals, &perm, expected_shape, expected_vals, {kTensorrtExecutionProvider}, {7, 21});
 }
 
 // test to cover memcpy from single axis moving inwards path
